# TCT_TubeMakerCmds
# Initial code generated by XSI SDK Wizard

#Note:  All text beginning with a '#' character represents 
#		a comment in Python, and has no affect on how the
#		code is interpreted by XSI


#When you use Python, you can import definitions of 
#constant values that are used with XSI's scripting
#interface. This makes it a lot easier to work with.
import win32com.client
from win32com.client import constants



#-------------------------------------------------------------------
#-------------------------------------------------------------------


#This is the first function that XSI looks at when loading
#a potential plugin. It contains details about who wrote it,
#support information, as well as the important declarations
#on what the plugin contains.

def XSILoadPlugin( in_reg ):
	in_reg.Author = "Andy Nicholas"
	in_reg.Name = "TCT_TubeMakerCmds"
	in_reg.Email = ""
	in_reg.URL = ""
	in_reg.Major = 1
	in_reg.Minor = 0

	#These two commands are the main ones that we call from the pipe maker
	in_reg.RegisterCommand("TCT_AddTube","TCT_AddTube")
	in_reg.RegisterCommand("TCT_DeleteTube","TCT_DeleteTube")

	#These helper commands are smaller and provide convenient reuse of code
	in_reg.RegisterCommand("TCTHelper_IsValidTube","TCTHelper_IsValidTube")
	in_reg.RegisterCommand("TCTHelper_GetTubeStartObj","TCTHelper_GetTubeStartObj")
	in_reg.RegisterCommand("TCTHelper_GetTubeEndObj","TCTHelper_GetTubeEndObj")
	in_reg.RegisterCommand("TCTHelper_GetTubeMeshObj","TCTHelper_GetTubeMeshObj")
	in_reg.RegisterCommand("TCTHelper_GetTubeConstraint","TCTHelper_GetTubeConstraint")
	in_reg.RegisterCommand("TCTHelper_GetPreviousTube","TCTHelper_GetPreviousTube")
	in_reg.RegisterCommand("TCTHelper_GetNextTube","TCTHelper_GetNextTube")
	in_reg.RegisterCommand("TCTHelper_ConstrainToTube","TCTHelper_ConstrainToTube")
	in_reg.RegisterCommand("TCTHelper_RemoveConstraint","TCTHelper_RemoveConstraint")

	return True


#-------------------------------------------------------------------
#Command:
#	TCT_AddTube

#Parameters:
#	tubeFileName		The filename of the tube model to load as a reference model
#	prevTube			The name of the scene model that the new tube is to be placed after,
#						or empty if it's the start of a new tube network
#	replaceExisting		The new tube will replace prevTube instead of being added after it.
	
#Return Value:	
#	The newly create tube reference model

#Function:
#	Adds or inserts a tube into a sequence


#Command Registration	
def TCT_AddTube_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeFileName",constants.siArgumentInput,"")
	oArgs.Add("prevTubeName",constants.siArgumentInput,"")
	oArgs.Add("replaceExisting",constants.siArgumentInput, False)	
	
#Command Definition
def TCT_AddTube_Execute(tubeFileName, prevTubeName, replaceExisting):	
	if tubeFileName=="":
		return None

	#Make sure that any PPGs (property pages) don't appear when we create objects
	#otherwise, the user has to spend a lot of time closing them later
	#(We remember to turn this preference back on at the end of the command)
	autoInspect = Application.GetValue("preferences.Interaction.autoinspect")
	Application.SetValue("preferences.Interaction.autoinspect", False)	
	
	#This is where we load the tube as a reference model
	retCol = Application.ImportModel(tubeFileName, None, True)
	tubeModelObj = retCol('Value')

	#Restore the original user settings for the PPG creation popup behaviour	
	Application.SetValue("preferences.Interaction.autoinspect", autoInspect)
	
	#If it hasn't loaded properly, then exit
	if tubeModelObj==None:
		return None

	#If this is the first tube in the sequence then we don't need to do anything else
	if prevTubeName=="":
		return tubeModelObj
	
	#If the tube we've been given to attach the new one to doesn't have the
	#right objects, then exit	
	if not TCTHelper_IsValidTube_Execute(prevTubeName):
		return tubeModelObj		

	#If we're replacing the currently selected tube		
	if replaceExisting:

		#Delete the previous tube, and store the next tube in selTube
		prevTubeModelObj = TCTHelper_GetPreviousTube_Execute(prevTubeName)
		selTube = Application.TCT_DeleteTube(prevTubeName)

		#If there's no next tube then return
		if selTube==None:
			return tubeModelObj

		#if there's no previous tube then just add the next one
		#to the one we've just created
		if prevTubeModelObj==None:
			TCTHelper_ConstrainToTube_Execute(selTube.FullName, tubeModelObj.FullName, False)
			return tubeModelObj
		
		prevTubeName = selTube.FullName

	#Whether we're inserting or replacing, we need to constrain the next tube
	#to the tube we've just created
	nextTubeModelObj = TCTHelper_GetNextTube_Execute(prevTubeName)
	if nextTubeModelObj!=None:
		TCTHelper_RemoveConstraint_Execute(nextTubeModelObj.FullName,True)
		TCTHelper_ConstrainToTube_Execute(nextTubeModelObj.FullName, tubeModelObj.FullName, False)
	
	#Finally, constrain the new tube to the previous tube 
	TCTHelper_ConstrainToTube_Execute(tubeModelObj.FullName, prevTubeName, False)
	return tubeModelObj
	


#-------------------------------------------------------------------
#Command:
#	TCT_DeleteTube

#Parameters:
#	deleteTube		The model name of the tube to delete
	
#Return Value:	
#	A reference to the previous tube model. If it doesn't exist, it'll return
#	the next tube model. Failing that, it'll return None

#Function:
#	Deletes a tube and attaches the tubes either side to each other


#Command Registration	
def TCT_DeleteTube_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("deleteTube",constants.siArgumentInput,"")

	
#Command Definition
def TCT_DeleteTube_Execute(deleteTube):
	#Get the tubes either side of the one we're about to delete
	nextTubeModelObj = TCTHelper_GetNextTube_Execute(deleteTube)
	prevTubeModelObj = TCTHelper_GetPreviousTube_Execute(deleteTube)

	#Reconstrain tubes either side
	if nextTubeModelObj!=None:
		
		#Remove constraint from next tube first
		TCTHelper_RemoveConstraint_Execute(nextTubeModelObj.FullName, False)		

		#Handle the case where we're deleting the first tube in the chain
		#by moving the next tube to the position of the first tube
		if prevTubeModelObj==None:
			nextRoot = TCTHelper_GetTubeStartObj_Execute(nextTubeModelObj.FullName)
			curRoot = TCTHelper_GetTubeStartObj_Execute(deleteTube)
			nextRoot.Kinematics.Global.PutTransform2(None, curRoot.Kinematics.Global.GetTransform2(None))
			Application.DeleteObj("B:"+deleteTube)
			
			return nextTubeModelObj

		#Delete the tube 
		Application.DeleteObj("B:"+deleteTube)
		
		#Attach the tubes either side of the one we just deleted
		TCTHelper_ConstrainToTube_Execute(nextTubeModelObj.FullName, prevTubeModelObj.FullName, True)
		
		constraint = Application.TCTHelper_GetTubeConstraint(nextTubeModelObj.FullName)		

		#Here we zero out the constraint rotational offsets except for the Z rotation
		constraint.Parameters("rotx").Value = 0
		constraint.Parameters("roty").Value = 0

		constraint.Parameters("posx").Value = 0
		constraint.Parameters("posy").Value = 0
		constraint.Parameters("posz").Value = 0						

	else:
		Application.DeleteObj("B:"+deleteTube)

	return prevTubeModelObj
	

	
#-------------------------------------------------------------------
#-------------------------------------------------------------------

# Helper function
def TCTHelper_GetSceneObject(objName):
	#This illustrates one of the fastest ways to obtain a reference to
	#an XSI object when identifying it by name
	col = win32com.client.Dispatch("XSI.Collection")
	try:
		col.SetAsText(objName)
		return col(0)		
	except:
		pass
	return None

	
	
	
#-------------------------------------------------------------------
#Command:
#	TCTHelper_IsValidTube

#Parameters:
#	tubeModelName	The name of the tube model
	
#Return Value:	
#	True if the model is a valid tube (i.e. it has the correctly 
#	named objects), false otherwise

#Function:
#	Validates a tube model


#Command Registration	
def TCTHelper_IsValidTube_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeModelName",constants.siArgumentInput,"")
		
#Command Definition	
def TCTHelper_IsValidTube_Execute(tubeModelName):
	#Check the tube model isn't actually the scene root
	if tubeModelName==Application.ActiveSceneRoot.Name:
		return False

	#Check the other objects exist
	if TCTHelper_GetTubeStartObj_Execute(tubeModelName)==None:
		return False
	if TCTHelper_GetTubeEndObj_Execute(tubeModelName)==None:
		return False
	if TCTHelper_GetTubeMeshObj_Execute(tubeModelName)==None:
		return False
	return True

	
#-------------------------------------------------------------------
#Command:
#	TCTHelper_GetTubeStartObj

#Parameters:
#	tubeModelName	The name of the tube model
	
#Return Value:	
#	A reference to the tube's start null

#Function:
#	Obtains a reference to the tube's start null


#Command Registration	
def TCTHelper_GetTubeStartObj_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)
	
	oArgs = oCmd.Arguments
	oArgs.Add("tubeModelName",constants.siArgumentInput,"")
		
#Command Definition	
def TCTHelper_GetTubeStartObj_Execute(tubeModelName):
	return TCTHelper_GetSceneObject(tubeModelName+".Tube_Start")
	
	

	
#-------------------------------------------------------------------
#Command:
#	TCTHelper_GetTubeEndObj

#Parameters:
#	tubeModelName	The name of the tube model
	
#Return Value:	
#	A reference to the tube's end null

#Function:
#	Obtains a reference to the tube's end null


#Command Registration	
def TCTHelper_GetTubeEndObj_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeModelName",constants.siArgumentInput,"")
		
#Command Definition	
def TCTHelper_GetTubeEndObj_Execute(tubeModelName):
	return TCTHelper_GetSceneObject(tubeModelName+".Tube_End")

	
#-------------------------------------------------------------------
#Command:
#	TCTHelper_GetTubeMeshObj

#Parameters:
#	tubeModelName	The name of the tube model
	
#Return Value:	
#	A reference to the tube's mesh object

#Function:
#	Obtains a reference to the tube's mesh object


#Command Registration	
def TCTHelper_GetTubeMeshObj_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeModelName",constants.siArgumentInput,"")
		
#Command Definition		
def TCTHelper_GetTubeMeshObj_Execute(tubeModelName):
	return TCTHelper_GetSceneObject(tubeModelName+".Tube_Mesh")

#-------------------------------------------------------------------
#Command:
#	TCTHelper_GetTubeConstraint

#Parameters:
#	tubeModelName	The name of the tube model
	
#Return Value:	
#	A reference to the tube's constraint

#Function:
#	Obtains a reference to the tube's constraint


#Command Registration	
def TCTHelper_GetTubeConstraint_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeModelName",constants.siArgumentInput,"")
		
#Command Definition		
def TCTHelper_GetTubeConstraint_Execute(tubeModelName):
	#Get the null object that is being constrained 
	startObj = TCTHelper_GetSceneObject(tubeModelName+".Tube_Start")
	if startObj==None:
		return None

	#Get the constraint using the constraint collection of the Kinematics
	constraintCol = startObj.Kinematics.Constraints
	if constraintCol.Count>0:
		return constraintCol(0)
	return None

	
	
#-------------------------------------------------------------------
#Command:
#	TCTHelper_GetPreviousTube

#Parameters:
#	tubeModelName	The name of the tube model
	
#Return Value:	
#	A reference to the tube's previous attached tube, None otherwise

#Function:
#	Obtains a reference to the tube's previous attached tube


#Command Registration	
def TCTHelper_GetPreviousTube_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeModelName",constants.siArgumentInput,"")
		
#Command Definition	
def TCTHelper_GetPreviousTube_Execute(tubeModelName):
	#Sometimes Python doesn't always return references to objects correctly.
	#This is a bug to do with the implementation of Python. So in the case
	#of getting the constraint, we're calling the command via the XSI command
	#mechanism instead, as it seems to work properly.
	constraint = Application.TCTHelper_GetTubeConstraint(tubeModelName)
	if constraint==None:
		return None

	#This returns the first object that is constraining (should only ever
	#be one object in this situation)
	return constraint.Constraining(0).Model

	
#-------------------------------------------------------------------
#Command:
#	TCTHelper_GetNextTube

#Parameters:
#	tubeModelName	The name of the tube model
	
#Return Value:	
#	A reference to the tube's next attached tube, None otherwise

#Function:
#	Obtains a reference to the tube's next attached tube


#Command Registration	
def TCTHelper_GetNextTube_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeModelName",constants.siArgumentInput,"")
		
#Command Definition			
def TCTHelper_GetNextTube_Execute(tubeModelName):
	#Find the object constrained to the end null of this tube 
	endObj = TCTHelper_GetTubeEndObj_Execute(tubeModelName)
	
	retcol = Application.GetConstrainedObjects(endObj)
	col = retcol('OutputObjs')
	
	#Return the first valid tube that is constrained to the null
	for item in col:
		if TCTHelper_IsValidTube_Execute(item.Model.FullName):
			return item.Model
	return None



#-------------------------------------------------------------------
#Command:
#	TCTHelper_ConstrainToTube

#Parameters:
#	tubeName_ToConstrain	The name of the tube model to constrain
#	tubeName_ToConstrainTo	The name of the tube model to constrain to
	
#Return Value:	
#	None

#Function:
#	Constrains one tube to another


#Command Registration	
def TCTHelper_ConstrainToTube_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeName_ToConstrain",constants.siArgumentInput,"")
	oArgs.Add("tubeName_ToConstrainTo",constants.siArgumentInput,"")
	oArgs.Add("use_compensation",constants.siArgumentInput,False)	
		
#Command Definition		
def TCTHelper_ConstrainToTube_Execute(tubeName_ToConstrain, tubeName_ToConstrainTo, use_compensation):
	#Get the two objects from the two pipes that are to be constrained to each other
	startObj = TCTHelper_GetTubeStartObj_Execute(tubeName_ToConstrain)
	endObj = TCTHelper_GetTubeEndObj_Execute(tubeName_ToConstrainTo)

	#Make sure that any PPGs (property pages) don't appear when we create objects
	#otherwise, the user has to spend a lot of time closing them later
	#(We remember to turn this preference back on at the end of the command)
	autoInspect = Application.GetValue("preferences.Interaction.autoinspect")
	Application.SetValue("preferences.Interaction.autoinspect", False)
	
	#Create the constraint 
	Application.ApplyCns("Pose", startObj, endObj, use_compensation)

	#Restore the original user settings for the PPG creation popup behaviour	
	Application.SetValue("preferences.Interaction.autoinspect", autoInspect)


#-------------------------------------------------------------------
#Command:
#	TCTHelper_RemoveConstraint

#Parameters:
#	tubeModelName	The name of the tube model
#	resetXForm		An option to reset the tube transform or leave it where it is
	
#Return Value:	
#	None

#Function:
#	Removes the constraint from a tube


#Command Registration	
def TCTHelper_RemoveConstraint_Init( ctxt ):
	oCmd = ctxt.Source
	oCmd.Description = ""
	oCmd.ReturnValue = True
	oCmd.SetFlag(constants.siNoLogging)

	oArgs = oCmd.Arguments
	oArgs.Add("tubeModelName",constants.siArgumentInput,"")
	oArgs.Add("resetXForm",constants.siArgumentInput,"")
		
#Command Definition	
def TCTHelper_RemoveConstraint_Execute(tubeModelName, resetXForm):
	
	#Get the start null and the constraint 
	startObj = TCTHelper_GetTubeStartObj_Execute(tubeModelName)
	cns = TCTHelper_GetTubeConstraint_Execute(tubeModelName)
	
	#Store current global transform to keep it in current
	#location after constraint is deleted (unless it is reset below)
	xform = startObj.Kinematics.Global.Transform
		
	#Delete the constraint 
	Application.DeleteObj(cns)
	
	#Reset the transform if required
	if resetXForm:
		xform.SetIdentity()
		
	startObj.Kinematics.Local.Transform = xform
